#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
mbt_02.py
"""

import sys
import serial
from time import sleep
from crc16c import calcByte,calcString,hexCRC,checkModbusWithCrc,readHex
from threading import Timer
from key_getter import KeyGetter # Tan sols funciona amb entorns amb GUI. Està basat en curses

class ModBus():
	def __init__(self, parent=None):
		
		if len( sys.argv ) == 2 :
			szPort = sys.argv[1]
			nBauds = 115200
		else :
			if len( sys.argv ) == 3 :
				szPort = sys.argv[1]
				nBauds = int(sys.argv[2])
			else :
				szPort = "/dev/ttyUSB0"
				nBauds = 115200

		port = szPort
		baudrate = nBauds
		
		print("Baudrate: %d at %s"%(baudrate,port))
		self.ser = 0
		self.t = 0
		
		try: 	
			self.ser = serial.Serial(
				port,
				baudrate,
				timeout=0,
				parity=serial.PARITY_NONE,
				stopbits=serial.STOPBITS_TWO,
				bytesize=serial.EIGHTBITS
			)
			
			self.nSegonsTimer = 2
			self.t = Timer(self.nSegonsTimer,self.temporitzador)
			self.t.start()
			
			k = KeyGetter()
			self.bLoop = True
			while self.bLoop == True:
				if k.kbhit():
					cKey = repr(k.getch(False))
					self.processa(cKey[1])
				sleep(0.1)
			self.bye()

		except serial.SerialException as e:
			print ("%s port is not connected" % szPort)
			sys.exit(-1)

		except KeyboardInterrupt:
			print ("\nAdéu!")
			self.t.cancel()		
			sys.exit(0)


	def processa(self,c):
		if c == 'q' or c == 'Q':
			self.bLoop = False
			return

	def bye(self):
		if self.t:
			self.t.cancel()	
		if self.ser:
			self.ser.close()
		
	def __del__(self):
		self.bye()
					
	def temporitzador(self):
		print("Crida al temporitzador")
		self.t = Timer(self.nSegonsTimer,self.temporitzador)
		self.t.start()
			
if __name__ == '__main__':

	mb = ModBus()
