#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
mbt_07.py
"""

import sys
import serial
from time import sleep
from crc16c import calcByte,calcString,hexCRC,checkModbusWithCrc,readHex
from threading import Timer
from key_getter import KeyGetter # Tan sols funciona amb entorns amb GUI. Està basat en curses

class ModBus():
	def __init__(self, parent=None):
		
		if len( sys.argv ) == 2 :
			szPort = sys.argv[1]
			nBauds = 115200
		else :
			if len( sys.argv ) == 3 :
				szPort = sys.argv[1]
				nBauds = int(sys.argv[2])
			else :
				szPort = "/dev/ttyUSB0"
				nBauds = 115200

		port = szPort
		baudrate = nBauds
		
		print("Baudrate: %d at %s"%(baudrate,port))
		self.ser = 0
		self.t = 0
		
		try: 	
			self.ser = serial.Serial(
				port,
				baudrate,
				timeout=0,
				parity=serial.PARITY_NONE,
				stopbits=serial.STOPBITS_TWO,
				bytesize=serial.EIGHTBITS
			)
			
			self.nSegonsTimer = 0.2
			self.t = Timer(self.nSegonsTimer,self.temporitzador)
			self.t.start()
			
			k = KeyGetter()
			self.bLoop = True
			while self.bLoop == True:
				if k.kbhit():
					cKey = repr(k.getch(False))
					self.processaTecla(cKey[1])
				sleep(0.1)
			self.bye()

		except serial.SerialException as e:
			print ("%s port is not connected" % szPort)
			sys.exit(-1)

		except KeyboardInterrupt:
			print ("\nAdéu!")
			self.t.cancel()		
			sys.exit(0)

	def writeCommand(self,command):
		hCRC = hexCRC(command)
		addedCRC="%c%c"%(hCRC>>8,hCRC&0xFF)
		szAddedCRC="%2X%2X"%(hCRC>>8,hCRC&0xFF)
		command += addedCRC
		#readHex(stMB)
		#print("")

		for cmd_byte in command:
			hex_byte = ("{0:02X}".format(ord(cmd_byte)))
			print (hex_byte,end='')
			self.ser.write(bytearray.fromhex(hex_byte))
		print("")		

	def lecturaMB(self,q):
		print("%s" % q)
		if q == 'B5':
			print("Lectura B5")
			#01 04 31 03 00 01
			stMB = "%c%c%c%c%c%c"%(0x01,0x04,0x31,0x03,0x00,0x01)
			self.writeCommand(stMB)

		if q == '0x3002':
			print("Lectura 0x3002")
			#01 04 30 02 00 01
			stMB = "%c%c%c%c%c%c"%(0x01,0x04,0x30,0x02,0x00,0x01)
			self.writeCommand(stMB)
			
		if q == '0x3003':
			print("Lectura 0x3003")
			#01 04 30 03 00 01
			stMB = "%c%c%c%c%c%c"%(0x01,0x04,0x30,0x03,0x00,0x01)
			self.writeCommand(stMB)

		if q == '0x3104':
			print("Lectura 0x3104")
			#01 04 31 04 00 01
			stMB = "%c%c%c%c%c%c"%(0x01,0x04,0x31,0x04,0x00,0x01)
			self.writeCommand(stMB)

		if q == 'SoC':
			print("Lectura 0x311A - SoC")
			#01 04 31 1A 00 01
			stMB = "%c%c%c%c%c%c"%(0x01,0x04,0x31,0x1A,0x00,0x01)
			self.writeCommand(stMB)


		if q == 'whoAmI':
			#F8 45 00 01 01 F8 89 BE
			stMB = "%c%c%c%c%c%c"%(0xF8,0x45,0x00,0x01,0x01,0xF8)
			self.writeCommand(stMB)

		if q == 'loadControllingMode':
			print("Load Controlling Mode")
			#01 03 90 3D 00 01
			stMB = "%c%c%c%c%c%c"%(0x01,0x03,0x90,0x3d,0x00,0x01)
			self.writeCommand(stMB)

		if q == 'manualControl':
			print("Manual Control")
			#01 06 90 3D 00 00
			stMB = "%c%c%c%c%c%c"%(0x01,0x06,0x90,0x3d,0x00,0x00)
			self.writeCommand(stMB)

		if q == 'lightOnOff':
			print("Light ON/OFF")
			#01 06 90 3D 00 01
			stMB = "%c%c%c%c%c%c"%(0x01,0x06,0x90,0x3d,0x00,0x01)
			self.writeCommand(stMB)

		if q == 'defaultInManual':
			print("Defaukt in Manual Mode")
			#01 03 90 6A 00 01
			stMB = "%c%c%c%c%c%c"%(0x01,0x03,0x90,0x3d,0x00,0x01)
			self.writeCommand(stMB)

		if q == 'defaultOff':
			print("Default OFF")
			#01 06 90 6a 00 00
			stMB = "%c%c%c%c%c%c"%(0x01,0x06,0x90,0x6a,0x00,0x00)
			self.writeCommand(stMB)

		if q == 'defaultOn':
			print("Default ON")
			#01 06 90 6a 00 01
			stMB = "%c%c%c%c%c%c"%(0x01,0x06,0x90,0x6a,0x00,0x01)
			self.writeCommand(stMB)

		if q == 'manualControlTheLoadOn':
			print("Manual Control The Load ON")
			#01 05 00 02 ff 00
			stMB = "%c%c%c%c%c%c"%(0x01,0x05,0x00,0x02,0xff,0x00)
			self.writeCommand(stMB)

		if q == 'manualControlTheLoadOff':
			print("Manual Control The Load OFF")
			#01 05 00 02 00 00
			stMB = "%c%c%c%c%c%c"%(0x01,0x05,0x00,0x02,0x00,0x00)
			self.writeCommand(stMB)

		if q == 'turnOff':
			print("Turn OFF")
			#01 05 00 06 00 00
			stMB = "%c%c%c%c%c%c"%(0x01,0x05,0x00,0x06,0x00,0x00)
			self.writeCommand(stMB)

		if q == 'turnOn':
			print("Turn ON")
			#01 05 00 06 ff 00
			stMB = "%c%c%c%c%c%c"%(0x01,0x05,0x00,0x06,0xff,0x00)
			self.writeCommand(stMB)

		if q == 'readLedRatedCurrent':
			print("C30 - 9078) Read Led Rated Current")
			#01 03 90 78 00 01
			stMB = "%c%c%c%c%c%c"%(0x01,0x03,0x90,0x78,0x00,0x01)
			self.writeCommand(stMB)

		if q == 'writeHalfLedRatedCurrent':
			print("C30 - 9078) Write Half Led Rated Current")
			#01 10 90 78 00 01 02 00 12
			stMB = "%c%c%c%c%c%c%c%c%c"%(0x01,0x10,0x90,0x78,0x00,0x01,0x02,0x00,0x12)
			self.writeCommand(stMB)

		if q == 'writeFullLedRatedCurrent':
			print("C30 - 9078) Write Full Led Rated Current")
			#01 10 90 78 00 01 02 00 23
			stMB = "%c%c%c%c%c%c%c%c%c"%(0x01,0x10,0x90,0x78,0x00,0x01,0x02,0x00,0x23)
			self.writeCommand(stMB)

			
	def processaTecla(self,c):
		if c == 'q' or c == 'Q':
			self.bLoop = False
			return
		if c == '0':
			self.lecturaMB('whoAmI')
		if c == '1':
			self.lecturaMB('B5')
		if c == '2':
			self.lecturaMB('0x3002')
		if c == '3':
			self.lecturaMB('0x3003')
		if c == '4':
			self.lecturaMB('0x3104')
		if c == 'r':
			self.lecturaMB('loadControllingMode')
		if c == 'M':
			self.lecturaMB('manualControl')
		if c == 'L':
			self.lecturaMB('lightOnOff')
		if c == 'R':
			self.lecturaMB('defaultInManual')
		if c == 'F':
			self.lecturaMB('defaultOff')
		if c == 'N':
			self.lecturaMB('defaultOn')
		if c == 'm':
			self.lecturaMB('manualControlTheLoadOn')
		if c == 'f':
			self.lecturaMB('manualControlTheLoadOff')
		if c == 't':
			self.lecturaMB('turnOff')
		if c == 'T':
			self.lecturaMB('turnOn')
		if c == 's':
			self.lecturaMB('SoC')
		if c == 'l':
			self.lecturaMB('readLedRatedCurrent')
		if c == '5':
			self.lecturaMB('writeHalfLedRatedCurrent')
		if c == '6':
			self.lecturaMB('writeFullLedRatedCurrent')

	def processa(self,sz,nQ):
		#self.bConn = 0
		#print ("Rebut: %s"%sz)
		print ("He rebut %d bytes" % nQ)
		#readHex(sz)
		for b in sz:
			hex_byte = ("{0:02X}".format(b))
			print (hex_byte,end='')		
		print("")

	def bye(self):
		if self.t:
			self.t.cancel()	
		if self.ser:
			self.ser.close()
		
	def __del__(self):
		self.bye()
					
	def temporitzador(self):
		data = self.ser.read(1)
		n = self.ser.inWaiting()
		if n:
			n = 1 + n
			data = data + self.ser.read(n)
			print("n: %d" % n)
		if len(data):
			self.processa(data,n)
			
		self.t = Timer(self.nSegonsTimer,self.temporitzador)
		self.t.start()
			
if __name__ == '__main__':

	mb = ModBus()
